#***************************************************************************
#*   Copyright (C) 2006 by Andr Lindhjem <belgarat@sdf.lonestar.org>,     *
#*                         Kjetil Holien <kjetil.holien@gmail.com>,        *
#*                         Terje Risa <terje.risa@gmail.com> &             *
#*                         yvind Nerbrten <oyvind@nerbraaten.com>        *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 2 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
#***************************************************************************

module DaisyAudio
	require "gst"
	
	Gst.init
	
	# the AudioEngine class handles the audio-part of the dailsyplayer.
	#
	class AudioEngine
	
		# This method plays a file.
		# * file - a string path to the file we want to play
		#
		def play (file)
			@audio		= Gst::Pipeline.new("pipeline")
			@source		= Gst::ElementFactory.make("filesrc")
			@decoder 	= Gst::ElementFactory.make("spider")	# spider figures out what decoder we need for us.
			@sink = Gst::ElementFactory.make("osssink")
			@source.location = file

			@audio.add(@source, @decoder, @sink)
			@source >> @decoder >> @sink
			puts "playing #{file}"
			@audio.play
			while @audio.iterate do end
			@audio.stop
		end

	
	# These are our private functions. These should only be used by objects of class 
	#
	private
	
		# Based on the argument, this method will make an audio sink and return it.
		# If the argument is invalid, it defaults to aRts.
		# Returns the newly created element.
		#
		def createSink (audioOutput)
			case audioOutput
			when "arts"	# We want to play it using oss.
				sink = Gst::ElementFactory.make("artsdsink")
			when "esd"	# We want to play it using oss.
				sink = Gst::ElementFactory.make("esdsink")
			when "alsa"	# We want to play it using alsa.
				sink = Gst::ElementFactory.make("alsasink")
			when "oss"	# We want to play it using oss.
				sink = Gst::ElementFactory.make("osssink")
			else
				# We prints out a warning and chooses a default output.
				# We use aRts as it's now default in skolelinux, but we might need to change this.
				puts "Audio output %s is not supported! Defaulting to: aRts."
				sink = Gst::ElementFactory.make("artsdsink")
			end
			return sink
		end
	
		# This local method links the source, decoder and sink elements.
		# If the link doesn't succeed, an error-message are printed and the program exits.
		def link (source, decoder, sink)
			if ((link = source >> decoder >> sink) == nil)
				puts "Error linking gstreamer components!"
				puts "Exiting!"
				exit!
			end
		end
	end
	
end
